package fi.jyu.mit.fxgui;

import java.util.Optional;

import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.ButtonBar.ButtonData;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Control;
import javafx.scene.control.TextInputDialog;
import javafx.scene.control.Tooltip;

/**
 * @author vesal
 * @version 31.12.2015
 * Joukko valmiita dialogeja helpottamaan yksinkertaisia tehtäviä. 
 */
public class Dialogs {
 
    /**
     * Rajapinta dialogin asettamisen takaisinkutsulle
     */
    public interface AlertSetterInterface {
        /**
         * @param alert mille dialogille asetetaan
         */
        public void set(Alert alert); 
    }
    
    
    /**
     * Rajapinta dialogin asettamisen takaisinkutsulle
     */
    public interface TextInputDialogSetterInterface {
        /**
         * @param dlg mille dialogille asetetaan
         */
        public void set(TextInputDialog dlg); 
    }
    
    
    /**
     * Asetetan tietylle kontrollille Tooltip
     * @param node mikkle kontrollille asetetaan
     * @param tip mikä viesti
     * @return luotu Tooltip tai null jos ei viestiä
     */
    public static Tooltip setToolTipText(Control node, String tip) {
        if ( tip == null || tip.isEmpty() ) {
            node.setTooltip(null);
            return null;
        }
        Tooltip tooltip = new Tooltip();
        tooltip.setText(tip);
        node.setTooltip(tooltip);
        return tooltip;
    }
    
    
    /**
     * Näytetään viesti käyttäjälle ja odotetaan kuittausta
     * @param message näytettävä viesti
     * @param alertSetter kutsu, jolla voidaan asettaa arvoja ennen näyttämistä
     */
    public static void showMessageDialog(String message, AlertSetterInterface alertSetter) {
        Alert alert = new Alert(AlertType.INFORMATION);
        alert.setTitle("Huomautus");
        alert.setHeaderText(null);
        alert.setContentText(message);
        alert.getDialogPane().setPrefSize(200, -1);
        if ( alertSetter != null ) alertSetter.set(alert);
        alert.showAndWait();
    }

    
    /**
     * Näytetään viesti käyttäjälle ja odotetaan kuittausta
     * @param message näytettävä viesti
     */
    public static void showMessageDialog(String message) {
        showMessageDialog(message, null);
    }

    
    /**
     * Näytetään dialogi, joka kysyy merkkijonon
     * @param prompt teksti joka näytetään kysymykseksi
     * @param def arvo joka on syötön pohjana
     * @param setter kutsu, jolla voidaan asettaa arvoja ennen näyttämistä
     * @return null mikäli painetaan Cancel, muuten käyttäjän antama syöte.
     */
    public static String showInputDialog(String prompt, String def, TextInputDialogSetterInterface setter) {
        TextInputDialog dialog = new TextInputDialog(def);
        dialog.setHeaderText(null);
        dialog.setTitle("Vastaa");
        dialog.setContentText(prompt + ":");
        if ( setter != null ) setter.set(dialog);
        Optional<String> answer = dialog.showAndWait();
        return answer.isPresent() ? answer.get() : null;
    }


    /**
     * Näytetään dialogi, joka kysyy merkkijonon
     * @param prompt teksti joka näytetään kysymykseksi
     * @param def arvo joka on syötön pohjana
     * @return null mikäli painetaan Cancel, muuten käyttäjän antama syöte.
     */
    public static String showInputDialog(String prompt, String def) {
        return showInputDialog(prompt, def, null);         
    }
    
    
    /**
     * Näytetään dialogi, jossa itse valittava otsikko, kysymysteksti
     * ja kaksi itse valittavaa painiketta joista 1. on oletus ja toinen cancel
     * @param title dialogin otsikko
     * @param question dislogissa oleva kysymys
     * @param buttonYes kyllä painikkeen teksti
     * @param buttonNo ei painikkeen teksti
     * @return true jos painetaan 1. painiketta ja false jos toista
     * @param alertSetter kutsu, jolla voidaan asettaa arvoja ennen näyttämistä
     */
    public static boolean showQuestionDialog(String title, String question,  
                        String buttonYes, String buttonNo,
                        AlertSetterInterface alertSetter) {
        Alert alert = new Alert(AlertType.CONFIRMATION);
        alert.setTitle(title);
        alert.setHeaderText(null);
        alert.setContentText(question);

        ButtonType buttonTypeYes = new ButtonType(buttonYes, ButtonData.OK_DONE);
        ButtonType buttonTypeCancel = new ButtonType(buttonNo, ButtonData.CANCEL_CLOSE);

        alert.getButtonTypes().setAll(buttonTypeYes, buttonTypeCancel);

        if ( alertSetter != null ) alertSetter.set(alert);
        Optional<ButtonType> result = alert.showAndWait();
        return result.get() == buttonTypeYes;
    }


    /**
     * Näytetään dialogi, jossa itse valittava otsikko, kysymysteksti
     * ja kaksi itse valittavaa painiketta joista 1. on oletus ja toinen cancel
     * @param title dialogin otsikko
     * @param question dislogissa oleva kysymys
     * @param buttonYes kyllä painikkeen teksti
     * @param buttonNo ei painikkeen teksti
     * @return true jos painetaan 1. painiketta ja false jos toista
     */
    public static boolean showQuestionDialog(String title, String question,  
                        String buttonYes, String buttonNo) {
        return showQuestionDialog(title, question, buttonYes, buttonNo, null);
    }

}
