package fi.jyu.mit.fxgui;

import javafx.beans.DefaultProperty;
import javafx.collections.ListChangeListener;
import javafx.geometry.Insets;
import javafx.scene.Node;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.layout.VBox;

/**
 * VBox with borders and ScrollPane inside.  Last added child is inside
 * ScrollPane.  Others are after that.
 * @author vesal
 * @version 4.2.2017
 */
@DefaultProperty("content")
public class ScrollBorder extends VBox {

    private ScrollPane sp = new ScrollPane();
    private Label labelValitse; 
    
    /**
     * Initialize component
     */
    public ScrollBorder() {
        super();
        VBox vbox2 = this;
        vbox2.setPadding(new Insets(10, 10, 10, 10));
        VBox.setMargin(vbox2, new Insets(5, 20, 20, 20));
        vbox2.setStyle("-fx-border-color: black;");

        labelValitse = new Label("Valitse:");
        super.getChildren().add(labelValitse);

        sp.getStyleClass().add("edge-to-edge");
        super.getChildren().add(sp);
        
        getChildren().addListener(new ListChangeListener<Node>() {
            // @SuppressWarnings("synthetic-access")
            @Override
            public void onChanged(Change<? extends Node> c) {
                if (!c.next()) return;
                for (Node item: c.getAddedSubList())
                    sp.setContent(item);
            }
        });
    }
    
    
    /**
     * @return ScrollPane
     */
    public ScrollPane getScrollPane() {
        return sp;
    }
    
    
    /**
     * @param sp new scrollPane
     */
    public void setScrollPane(ScrollPane sp) {
        this.sp = sp;
    }
    
    
    /**
     * @param node the box to scroll
     */
    public void setContent(Node node) {
        sp.setContent(node);
    }

    
    /**
     * @return current content
     */
    public final Node getContent() {
        return sp.getContent();
    }
    
    
    
    /**
     * @param text caption for component
     */
    public void setCaption(String text) {
        labelValitse.setText(text);
    }
    
    
    /**
     * @return current caption
     */
    public String getCaption() {
        return labelValitse.getText();
    }
    
}
